const util = require('util');
const dateFormat = require('date-format');
const ALY = require('aliyun-sdk');

function level(loggingEvent) {
    return loggingEvent.level.levelStr;
}

function message(loggingEvent) {
    return util.format(...loggingEvent.data);
}

function location(loggingEvent) {
    if (loggingEvent.callStack) {
        return `${loggingEvent.categoryName} ${loggingEvent.functionName}(${loggingEvent.fileName}:${loggingEvent.lineNumber}:${loggingEvent.columnNumber})`;
    }
    return loggingEvent.categoryName || '';
}

function time(loggingEvent, timeFormat, timezoneOffset) {
    return dateFormat(timeFormat, loggingEvent.startTime);
}

module.exports.configure = function (config, layouts) {
    if (!config.accessKeyId ||
        !config.secretAccessKey ||
        !config.endpoint ||
        !config.apiVersion ||
        !config.projectName ||
        !config.logStoreName) {
        console.error('config error');
        return;
    }

    const layout = config.layout ?
        layouts.layout(config.layout.type, config.layout) :
        layouts.layout;
    const timeFormat = config.timeFormat || 'yyyy-MM-ddThh:mmO';

    const sls = new ALY.SLS(config);
    const { projectName, logStoreName, topic } = config;

    const appender = function (loggingEvent) {
        appender.queue.push({
            time: Math.floor(loggingEvent.startTime.getTime() / 1000), //单位秒
            contents: [{
                key: 'level',
                value: level(loggingEvent)
            }, {
                key: 'time',
                value: time(loggingEvent, timeFormat)
            }, {
                key: 'message',
                value: message(loggingEvent)
            }, {
                key: 'location',
                value: location(loggingEvent)
            }, {
                key: 'log',
                value: layout(loggingEvent)
            }]
        });

    };

    appender.queue = [];
    appender.flushing = false;
    appender.flushTime = Date.now();
    appender.flushError = null;
    appender.flushQueue = [];
    appender.batchSize = config.batchSize || 4096;
    appender.lingerMs = config.lingerMs || 2000;

    appender.flush = function (size) {
        if (appender.flushing) {
            return;
        }
        if (!appender.flushError) {
            appender.flushQueue = appender.queue.splice(0, appender.batchSize);
        }
        if (appender.flushQueue.length > 0) {
            appender.flushing = true;
            sls.putLogs({
                projectName,
                logStoreName,
                logGroup: {
                    logs: appender.flushQueue,
                    topic
                }
            }, (err, data) => {
                appender.flushError = err;
                appender.flushTime = Date.now();
                appender.flushing = false;
                if (size === undefined) {
                    size = appender.batchSize;
                }
                if (!err && appender.queue.length > size) {
                    appender.flush();
                }
            });
        }
    };

    const flushId = setInterval(() => {
        if (appender.queue.length > appender.batchSize) {
            appender.flush();
        }
        if (Date.now() > appender.flushTime + appender.lingerMs) {
            appender.flush();
        }
    }, 1000);

    appender.shutdown = function (done) {
        clearInterval(flushId);
        appender.flush(0);
        if (done) {
            done();
        }
    };
    return appender;
};